import { initializeApp } from "firebase/app";

import {
  getDatabase,
  ref,
  update,
  onDisconnect,
  onValue,
  off,
  remove,
} from "firebase/database";

import { getAuth, signInAnonymously } from "firebase/auth";

let store;
const ECCO_DEBUG = document.cookie.indexOf("ecco") > -1;

function eccoLog(message, e) {
  if (ECCO_DEBUG) {
    console.log(message, e);
  }
}

async function getConfig(apiKey) {
  try {
    const response = await fetch(
      "https://www.ecco.xyz/api/v1/projects/" + apiKey,
      {
        mode: "cors",
        method: "GET",
        headers: {
          "Content-Type": "application/json",
          "X-Requested-With": "XMLHttpRequest",
        },
        redirect: "error",
      }
    );

    const data = await response.json();

    return data;
  } catch (e) {
    eccoLog("failed to fetch", e);
    return null;
  }
}

function isBot() {
  const bots = [
    "Googlebot",
    "Pinterestbot",
    "Chrome-Lighthouse",
    "AdsBot",
    "facebookexternalhit",
    "AhrefsBot",
    "bingbot",
    "applebot",
    "Google-Read-Aloud",
    "BingPreview",
    "node-fetch",
    "Go-http-client",
    "python-requests",
    "SemrushBot",
    "HeadlessChrome",
    "UptimeRobot",
    "YaBrowser",
    "Yowser",
    "Yandex",
    "Slurp",
    "Baidu",
    "MJ12",
    "DotBot",
    "msnbot",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3579.0 Safari/537.36",
  ];

  for (var i = 0; i < bots.length; i++) {
    if (navigator.userAgent.indexOf(bots[i]) > -1) {
      return true;
    } else {
      eccoLog("not", bots[i]);
    }
  }
}

async function getUser() {
  const auth = getAuth();
  return await signInAnonymously(auth);
}

function doThing(config, { userId, pageId, handleUpdate }) {
  const isDisconnected = false;
  let hasConnectedOnce = false;
  if (pageId) {
    if (pageId.indexOf("/products/") !== 0) {
      pageId = "/products/" + pageId;
    }
  } else {
    pageId =
      window.location.pathname === "/" ? "root" : window.location.pathname;
  }
  const location = `${encodeURIComponent(pageId)}`;

  const connectedRef = ref(store, ".info/connected");
  const sessionPath = ref(store, `${location}/views/${userId}`);
  const countRef = ref(store, location + "/view_count");
  let hasCountedMe = false;

  let isConnectedTimeout = null;
  const onWindowVisibilityChange = () => {
    if (document.hasFocus()) {
      eccoLog("setting from focus");
      update(sessionPath, { created_at: Date.now(), v: 2 });
    } else {
      eccoLog("removing from blur");
      remove(sessionPath);
    }
  };

  window.addEventListener("focus", onWindowVisibilityChange);
  window.addEventListener("blur", onWindowVisibilityChange);

  onValue(connectedRef, (snapshot) => {
    const isConnected = snapshot.val();

    if (isConnected) {
      eccoLog("update");
      update(sessionPath, { created_at: Date.now(), v: 2 });
      onDisconnect(sessionPath)
        .remove()
        .then(() => {
          eccoLog("onDisconnect set up");
        });

      hasConnectedOnce = true;
    } else {
      eccoLog("not connected");
    }
  });

  onValue(countRef, (snapshot) => {
    eccoLog("count ref", snapshot.val());
    let count = snapshot.val() || 0;
    if (!hasCountedMe) {
      count += 1;
    }
    eccoLog("calling back with", count);
    handleUpdate(count);
    hasCountedMe = true;
  });

  return () => {
    if (store) {
      try {
        eccoLog("detaching");
        window.removeEventListener("focus", onWindowVisibilityChange);
        window.removeEventListener("blur", onWindowVisibilityChange);
        remove(sessionPath);
        off(connectedRef);
        off(countRef);
      } catch (e) {
        eccoLog(e, "stop listening catch");
        // no-op
      }
    }
  };
}

function ViewerCount({
  apiKey,
  handleUpdate,
  element,
  minimumCount = 1,
  singularText = "",
  userId,
  pageId,
  pluralText = "${count} people viewing this page",
}) {
  let hasStoppedListening = false;
  let unsub;

  if (isBot()) {
    eccoLog("BOT!");
    return {
      stopListening: () => {},
    };
  }

  (async function init() {
    eccoLog("--- INIT ---");
    let config;

    if (!store) {
      eccoLog("--- NOT STORE ---");
      config = await getConfig(apiKey);

      if (!config) {
        eccoLog("--- NO CONFIG ---");
        return;
      }

      initializeApp({
        apiKey: config.firebase_api_key,
        databaseURL: `https://${config.firebase_name}.firebaseio.com`,
      });
      store = getDatabase();
    }

    const updateFn = element
      ? (count) => {
          let text = "";

          if (count > minimumCount) {
            text =
              count > 1
                ? pluralText.replace("${count}", count)
                : singularText.replace("${count}", count);
          }

          element.innerHTML = text;
          handleUpdate(count);
        }
      : handleUpdate;

    let user = null;
    if (!userId) {
      const { user } = await getUser();
      userId = user.uid;
      eccoLog("fb user", user.uid);
    } else {
      eccoLog("passed user", userId);
    }

    if (!hasStoppedListening) {
      unsub = doThing(config, {
        userId,
        pageId,
        handleUpdate: updateFn,
      });
    }
  })();

  return {
    stopListening: () => {
      eccoLog("stop listening");
      if (unsub) {
        unsub();
      } else {
        eccoLog("setting hasStoppedListening, no usnub");
        hasStoppedListening = true;
      }
    },
  };
}

export { ViewerCount };
